﻿

local moduleName = "Debuffs"



----------------------------
--      Localization      --
----------------------------

local L = AceLibrary("AceLocale-2.2"):new("BunchOfBars"..moduleName)

L:RegisterTranslations("enUS", function() return {
	[moduleName] = "Debuffs",

	["Text Color"] = true,
	["Change the text color."] = true
} end)



----------------------------------
--      Local Declaration      --
----------------------------------

local BS = LibStub("LibBabble-Spell-3.0"):GetLookupTable()

local candispel = {
	["WARRIOR"] = { },
	["ROGUE"] = { },
	["HUNTER"] = { },
	["MAGE"] = {
		["Curse"] = true
	},
	["WARLOCK"] = { },
	["DRUID"] = { 
		["Curse"] = true,
		["Poison"] = true
	},
	["PALADIN"] = {
		["Magic"] = true,
		["Poison"] = true,
		["Disease"] = true
	},
	["PRIEST"] = {
		["Magic"] = true,
		["Disease"] = true
	},
	["SHAMAN"] = {
		["Disease"] = true,
		["Poison"] = true
	}
}

do
	local class = select(2, UnitClass("player"))
	candispel = candispel[class] -- let's hope the rest of the candispel table is garbage collected after this
	candispel["none"] = true
end


local ignore = {
	-- debuffs here should never be dispelled
	[BS["Dreamless Sleep"]] = true,
	[BS["Greater Dreamless Sleep"]] = true,
	[BS["Major Dreamless Sleep"]] = true,

	[BS["Delusions of Jin'do"]] = true,

	[BS["Unstable Affliction"]] = true,

	["Upset Tummy"] = true, -- because it's very annoying

	-- debuffs here can be ignored
	["WARRIOR"] = {
		[BS["Ancient Hysteria"]] = true, -- Spirit reduced by 50%, Intellect reduced by 50%.
		[BS["Ignite Mana"]] = true, -- 400 mana burned every 3 sec.
		[BS["Tainted Mind"]] = true, -- Intellect reduced by 20%.
		[BS["Curse of Tongues"]] = true -- Casting speed reduced by 75%.
	},
	["ROGUE"] = {
		[BS["Ancient Hysteria"]] = true, -- Spirit reduced by 50%, Intellect reduced by 50%.
		[BS["Ignite Mana"]] = true, -- 400 mana burned every 3 sec.
		[BS["Tainted Mind"]] = true, -- Intellect reduced by 20%.
		[BS["Curse of Tongues"]] = true, -- Casting speed reduced by 75%.
		[BS["Silence"]] = true, -- Cannot cast spells.
		[BS["Sonic Burst"]] = true -- Cannot cast spells.
	},
	["HUNTER"] = {
	},
	["MAGE"] = {
	},
	["WARLOCK"] = {
	},
	["DRUID"] = {
	},
	["PALADIN"] = {
	},
	["PRIEST"] = {
	},
	["SHAMAN"] = {
	}
}

local agro = { }
local agrounit

-- localize these functions to speed up our main loop
local UnitClass = UnitClass
local UnitDebuff = UnitDebuff



----------------------------------
--      Module Declaration      --
----------------------------------

local plugin = BunchOfBars:NewModule(moduleName)

plugin.revision = tonumber(("$Revision: 56264 $"):match("%d+"))

plugin.options = {
	name = L[moduleName],
	args = {
		color = {
			type = "color",
			name = L["Text Color"],
			desc = L["Change the text color."],
			get = "GetSetColor",
			set = "GetSetColor"
		}
	}
}

plugin.defaultDB = {
	position = 1,

	color  = {
		r = 1,
		g = 1,
		b = 0
	},

	debuffs = {
		[BS["Water Tomb"]]				 = 1, -- Hydros
		[BS["Watery Grave"]]			 = 1, -- Morogrim
		[BS["Frost Blast"]]				 = 1, -- Kel'thuzad
		[BS["Garrote"]]					 = 1, -- Moroes
		[BS["Mortal Strike"]]			 = 1,
		[BS["Whirlwind"]]				 = 1, -- Leotheras the Blind
		[BS["Insidious Whisper"]]		 = 1, -- Leotheras the Blind
		[BS["Static Charge"]]			 = 1, -- Lady Vashj
		[BS["Panic"]]					 = 1, -- Lady Vashj Striders
		[BS["Wrath of the Astromancer"]] = 1, -- High Astromancer Solarian
		[BS["Conflagration"]]			 = 1, -- Kael'thas Sunstrider
		[BS["Remote Toy"]]				 = 1, -- Kael'thas Sunstrider
		[BS["Incinerate"]]				 = 1, -- Teron Gorefiend
		[BS["Shadow of Death"]]			 = 1, -- Teron Gorefiend
		[BS["Melt Armor"]]				 = 1, -- Al'ar
		[BS["Divine Wrath"]]			 = 1, -- Illidari Council
		[BS["Deadly Poison"]]			 = 1, -- Illidari Council
		[BS["Fatal Attraction"]]		 = 1, -- Mother Shahraz
		[BS["Vile Beam"]]				 = 1, -- Mother Shahraz
		[BS["Impaling Spine"]]			 = 1, -- High Warlord Naj'entus
		[BS["Doomfire"]]				 = 1, -- Archimonde
		[BS["Parasitic Shadowfiend"]]	 = 1, -- Illidan Stormrage
		[BS["Agonizing Flames"]]		 = 1, -- Illidan Stormrage
		["Shear"]						 = 1, -- Illidan Stormrage
		[BS["Dark Barrage"]]			 = 1, -- Illidan Stormrage
		["Creeping Paralysis"]			 = 1, -- Zul'jin
		["Grievous Throw"]				 = 1, -- Zul'jin
		["Bloodboil"]					 = 2, -- Gurtogg Bloodboil (having one of these debuffs is pretty normal here, 2 or more isn't)
		["Arcane Buffet"]				 = 10,
		["Corrupting Strike"]			 = 1,
		["Stomp"]						 = 1,
		["Burn"]						 = 1,
		["Encapsulate"]					 = 1,
		["Corrosion"]					 = 1,
		["Gas Nova"]					 = 1,
		["Flame Sear"]					 = 1,
		["Dark Strike"]					 = 1,
		["Dark Touched"]				 = 1,
		["Flame Touched"]				 = 1,
		[BS["Moonfire"]]				 = 1,
		["Fire Bloom"]					 = 1,
		["Living Bomb"]					 = 1,
		["Haunt"]						 = 1,
		["Blood Mirror"]				 = 1
	},

	buffs = {
		[BS["Flurry"]]				= true,
		[BS["Enrage"]]				= true,
		[BS["Rampage"]]				= true,
		[BS["Recklessness"]]		= true,
		[BS["Death Wish"]]			= true,
		[BS["Nature's Swiftness"]]	= true,
		[BS["Presence of Mind"]]	= true,
		[BS["Combustion"]]			= true,
		[BS["Slice and Dice"]]		= true,
		[BS["Avenging Wrath"]]		= true,
		[BS["Nightfall"]]			= true,
		[BS["Blood Fury"]]			= true,
		[BS["Rapid Fire"]]			= true
	}
}



----------------------------------
--      Module Functions        --
----------------------------------

function plugin:OnEnable()
	self:RegisterBucketEvent({"UNIT_AURA", "BunchOfBars_Agro"}, 0.5, "UpdateUnits")

	if self.core:HasModule("Agro") then
		agro     = self.core:GetModule("Agro").agro
		agrounit = self.core:GetModule("Agro").agrounit
	end
end


function plugin:OnCreate(frame)
	local buffbar = CreateFrame("Frame", nil, frame)
	buffbar:SetWidth(4 * (15 + 1))
	buffbar:SetHeight(18)

	return buffbar
end


function plugin:CreateBuff(buffbar, n)
	local t = buffbar:CreateTexture(nil, "ARTWORK")
	t:SetHeight(18)
	t:SetWidth(15)
	t:SetVertexColor(1, 1, 1, 0.9)
	t:SetTexCoord(0.2, 0.8, 0.1, 0.9)
	t:ClearAllPoints()
	t:SetPoint("LEFT", buffbar, "LEFT", (4 - n) * 16, 0)
	t:Hide()

	local font = buffbar:CreateFontString(nil, "OVERLAY")
	font:SetFontObject(GameFontNormalSmall)
	font:SetTextColor(self.db.profile.color.r, self.db.profile.color.g, self.db.profile.color.b, 1)
	font:ClearAllPoints()
	font:SetAllPoints(t)
	font:Hide()
	t.tfont = font

	buffbar[n] = t
end


function plugin:OnUpdate(frame, buffbar)
	local n = 1
	local class = select(2, UnitClass(frame.unit))

	for i = 1,32 do
		local name, _, texture, count, type = UnitDebuff(frame.unit, i)

		if not name then break end

		if not count or count < 1 then count = 1 end

		if (candispel[type] or (self.db.profile.debuffs[name] and count >= self.db.profile.debuffs[name])) and not (ignore[name] or ignore[class][name]) then
			if not buffbar[n] then
				self:CreateBuff(buffbar, n)
			end

			buffbar[n]:SetTexture(texture)
			buffbar[n]:SetVertexColor(1, 1, 1, 0.9)
			buffbar[n]:Show()

			if count and count > 0 then
				buffbar[n].tfont:SetText(count)
				buffbar[n].tfont:Show()
			else
				buffbar[n].tfont:Hide()
			end

			n = n + 1

			if n > 4 then break end
		end
	end

	if agro[frame.unit] and UnitExists(agrounit[frame.unit]) then
		for i = 1,32 do
			local name, _, texture = UnitBuff(agrounit[frame.unit], i)

			if not name then break end

			if (self.db.profile.buffs[name]) then
				if not buffbar[n] then
					self:CreateBuff(buffbar, n)
				end

				buffbar[n]:SetTexture(texture)
				buffbar[n]:SetVertexColor(1, 0.5, 0.5, 0.9)
				buffbar[n]:Show()
				buffbar[n].tfont:Hide()

				n = n + 1

				if n > 4 then break end
			end
		end
	end

	while buffbar[n] do
		buffbar[n]:Hide()
		buffbar[n].tfont:Hide()
		n = n + 1
	end
end



----------------------------------
--      Option Handlers         --
----------------------------------

function plugin:GetSetColor(r, g, b)
	if type(r) == "nil" then return self.db.profile.color.r, self.db.profile.color.g, self.db.profile.color.b end

	self.db.profile.color.r = r
	self.db.profile.color.g = g
	self.db.profile.color.b = b

	for _,frame in pairs(self.core.frames) do
		local buffbar, n = frame.parts[self.name], 1

		while buffbar[n] do
			buffbar[n].tfont:SetTextColor(self.db.profile.color.r ,self.db.profile.color.g, self.db.profile.color.b, 1)
			n = n + 1
		end
	end
end
